<?php

namespace Drupal\email_tfa\EventSubscriber;

use Drupal\Core\TempStore\PrivateTempStoreFactory;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Session\AccountInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpKernel\Event\GetResponseEvent;
use Symfony\Component\HttpKernel\KernelEvents;
use Drupal\Core\Url;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Messenger\MessengerInterface;

/**
 * Redirect user to Email TFA.
 */
class EmailTfaSubscriber implements EventSubscriberInterface {

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $account;

  /**
   * The route match object for the current page.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected $routeMatch;


  /**
   * The aggregator.settings config object.
   *
   * @var \Drupal\Core\Config\Config
   */
  protected $config;


  /**
   * The tempstore factory.
   *
   * @var \Drupal\Core\TempStore\PrivateTempStoreFactory
   */
  protected $tempStoreFactory;

  /**
   * The messenger.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * Constructs a new subscriber after login.
   *
   * @param \Drupal\Core\Session\AccountInterface $account
   *   The current user.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   The Route Match.
   * @param \Drupal\Core\TempStore\PrivateTempStoreFactory $temp_store_factory
   *   The factory for the temp store object.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger.
   */
  public function __construct(AccountInterface $account, ConfigFactoryInterface $config_factory, RouteMatchInterface $route_match, PrivateTempStoreFactory $temp_store_factory, MessengerInterface $messenger) {
    $this->account = $account;
    $this->config = $config_factory->get('email_tfa.settings');
    $this->routeMatch = $route_match;
    $this->tempStoreFactory = $temp_store_factory->get('email_tfa');
    $this->messenger = $messenger;

  }

  /**
   * Listen to kernel.request events and call emailTfaRedirection.
   */
  public static function getSubscribedEvents() {
    $events[KernelEvents::REQUEST][] = ['emailTfaRedirection'];
    return $events;
  }

  /**
   * Redirect pattern based url.
   *
   * @param \Symfony\Component\HttpKernel\Event\GetResponseEvent $event
   *   Response event.
   */
  public function emailTfaRedirection(GetResponseEvent $event) {
    $result = FALSE;
    $status = $this->config->get('status');
    // Load our pathways for this module.
    $tracks = $this->config->get('tracks');
    // We will exclude all rules coming from this setting,.
    $ignore_role = $this->config->get('ignore_role');
    // Check if pathway (Globally Enabled) selected && exclude roles selected.
    // Check if module is active from its settings and user is Authenticated.
    if ($status && $this->account->isAuthenticated()) {
      // User one status.
      $user_one = $this->config->get('user_one');
      $user = $this->account;
      // If user one is exclude and its the current one.
      if ($user_one == 1 && $user->id() == 1 && $tracks == 'globally_enabled') {
        $result = FALSE;
      }
      // Globally enabled pathway && some roles if selected will excluded.
      elseif ($tracks == 'globally_enabled' && !in_array_any($this->account->getRoles(), $ignore_role)) {
        // Check if user is NOT already verified.
        if ($this->tempStoreFactory->get('email_tfa_user_verify') == 0) {
          $result = TRUE;
        }
      }
      elseif ($tracks == 'optionally_by_users') {
        // @todo https://www.drupal.org/project/email_tfa/issues/3202328
        if (!empty(userCurrentStatus()) && userCurrentStatus() == 1 && $this->tempStoreFactory->get('email_tfa_user_verify') == 0) {
          $result = TRUE;
        }
      }

    }
    // Continue if one of the conditions above succeeds ( res = true ).
    if ($result) {
      if ($this->tempStoreFactory->get('email_tfa_send_mail') == "send_mail") {
        $this->tempStoreFactory->set('email_tfa_user_verify', 0);
        $this->tempStoreFactory->set('email_tfa_mail_created', time());
        $this->tempStoreFactory->set('email_tfa_otp_number', rand(1000, 9999));
        $this->tempStoreFactory->set('email_tfa_send_mail', "not_send_mail");
        \Drupal::service('plugin.manager.mail')->mail('email_tfa', 'send_email_tfa', $this->account->getEmail(), \Drupal::languageManager()->getDefaultLanguage()->getId(), [
          'user' => $this->account,
          'email_tfa' => $this->tempStoreFactory->get('email_tfa_otp_number'),
        ]);
      }
      $timeouts = $this->config->get('timeouts');
      // If time is passed/expired then redirects user to login page.
      if (time() - (int) $this->tempStoreFactory->get('email_tfa_mail_created') > $timeouts) {
        $request = \Drupal::request();
        $request->getSession()->clear();
        $url = Url::fromRoute('user.page')->toString();
        $response = new RedirectResponse($url);
        $response->send();
        exit;
      }
      // Pages that excluded from the redirect from module settings.
      $routes = $this->config->get('routes');
      $routes = array_map('trim', explode("\n", $routes));
      if (!in_array($this->routeMatch->getRouteName(), $routes)) {
        $url = Url::fromRoute('email_tfa.verifiy')->toString();
        $response = new RedirectResponse($url);
        $response->send();
        exit;
      }

    }

  }

}
